@@ -11,7 +11,12 @@ from .types import (
     HomeChargerTechnicalInfo,
     UserChargingStatus,
 )
-from .exceptions import ChargePointLoginError, ChargePointCommunicationException
+from .exceptions import (
+    ChargePointLoginError,
+    ChargePointCommunicationException,
+    ChargePointBaseException,
+    ChargePointInvalidSession,
+)
 from .session import ChargingSession
 from .constants import _LOGGER, V5_URL, MAP_URL
 
@@ -32,13 +37,27 @@ def _require_login(func):
         self = args[0]
         if not self._logged_in:
             raise RuntimeError("Must login to use ChargePoint API")
-        return func(*args, **kwargs)
+        try:
+            return func(*args, **kwargs)
+        except ChargePointCommunicationException as exc:
+            if exc.response.status_code == codes.unauthorized:
+                raise ChargePointInvalidSession(
+                    exc.response, "Session token has expired. Please login again!"
+                ) from exc
+            else:
+                raise
 
     return check_login
 
 
 class ChargePoint:
-    def __init__(self, username: str, password: str, app_version: str = "5.91.0"):
+    def __init__(
+        self,
+        username: str,
+        password: str,
+        app_version: str = "5.91.0",
+        session_token: str = "",
+    ):
         self._session = Session()
         self._app_version = app_version
         self._device_data = {
@@ -54,6 +73,20 @@ class ChargePoint:
         self._device_query_params = _dict_for_query(self._device_data)
         self._user_id = None
         self._logged_in = False
+        self._session_token = None
+
+        if session_token:
+            self._set_session_token(session_token)
+            self._logged_in = True
+            try:
+                account: ChargePointAccount = self.get_account()
+                self._user_id = str(account.user.user_id)
+                return
+            except ChargePointCommunicationException:
+                _LOGGER.warning(
+                    "Provided session token is expired, will attempt to re-login"
+                )
+                self._logged_in = False
 
         self.login(username, password)
 
@@ -65,6 +98,10 @@ class ChargePoint:
     def session(self) -> Session:
         return self._session
 
+    @property
+    def session_token(self) -> Optional[str]:
+        return self._session_token
+
     @property
     def device_data(self) -> dict:
         return self._device_data
@@ -88,20 +125,14 @@ class ChargePoint:
         }
         _LOGGER.debug("Attempting client login with user: %s", username)
         login = post(login_url, json=request, headers=headers)
+        _LOGGER.debug(login.cookies.get_dict())
+        _LOGGER.debug(login.headers)
 
         if login.status_code == codes.ok:
             req = login.json()
             self._user_id = req["user"]["userId"]
             _LOGGER.debug("Authentication success! User ID: %s", self._user_id)
-            self._session.headers = {
-                "cp-session-type": "CP_SESSION_TOKEN",
-                "cp-session-token": req["sessionId"],
-                # Data:       |------------------Token Data------------------||---?---||-Reg-|
-                # Session ID: rAnDomBaSe64EnCodEdDaTaToKeNrAnDomBaSe64EnCodEdD#D???????#RNA-US
-                "cp-region": req["sessionId"].split("#R")[1],
-                "user-agent": "ChargePoint/225 (iPhone; iOS 15.3; Scale/3.00)",
-            }
-            self._session.cookies.set("coulomb_sess", req["sessionId"])
+            self._set_session_token(req["sessionId"])
             self._logged_in = True
             return
 
@@ -125,8 +156,25 @@ class ChargePoint:
 
         self._session.headers = {}
         self._session.cookies.clear_session_cookies()
+        self._session_token = None
         self._logged_in = False
 
+    def _set_session_token(self, session_token: str):
+        try:
+            self._session.headers = {
+                "cp-session-type": "CP_SESSION_TOKEN",
+                "cp-session-token": session_token,
+                # Data:       |------------------Token Data------------------||---?---||-Reg-|
+                # Session ID: rAnDomBaSe64EnCodEdDaTaToKeNrAnDomBaSe64EnCodEdD#D???????#RNA-US
+                "cp-region": session_token.split("#R")[1],
+                "user-agent": "ChargePoint/225 (iPhone; iOS 15.3; Scale/3.00)",
+            }
+        except IndexError:
+            raise ChargePointBaseException("Invalid session token format.")
+
+        self._session_token = session_token
+        self._session.cookies.set("coulomb_sess", session_token)
+
     @_require_login
     def get_account(self) -> ChargePointAccount:
         _LOGGER.debug("Getting ChargePoint Account Details")
